package jdk.management.resource.internal.inst;

import java.nio.channels.ServerSocketChannel;
import java.net.InetSocketAddress;
import java.io.FileDescriptor;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import jdk.management.resource.internal.ResourceIdImpl;
import jdk.management.resource.internal.ApproverGroup;
import java.nio.channels.SocketChannel;
import jdk.internal.instrumentation.InstrumentationMethod;
import java.io.IOException;
import java.net.SocketAddress;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("sun.nio.ch.ServerSocketChannelImpl")
public final class ServerSocketChannelImplRMHooks
{
    private static ServerSocketChannelImplRMHooks.NativeDispatcher nd;
    
    public ServerSocketChannelImplRMHooks() {
    }
    
    @InstrumentationMethod
    public SocketAddress getLocalAddress() throws IOException {
        return this.getLocalAddress();
    }
    
    @InstrumentationMethod
    public SocketChannel accept() throws IOException {
        long request = 0L;
        long n = 0L;
        SocketChannel accept = null;
        ResourceIdImpl of = null;
        ResourceRequest approver = null;
        try {
            accept = this.accept();
            if (accept != null) {
                approver = ApproverGroup.SOCKET_OPEN_GROUP.getApprover(accept);
                of = ResourceIdImpl.of(this.getLocalAddress());
                try {
                    request = approver.request(1L, of);
                    if (request < 1L) {
                        try {
                            accept.close();
                        }
                        catch (IOException ex) {}
                        throw new IOException("Resource limited: too many open socket channels");
                    }
                }
                catch (ResourceRequestDeniedException cause) {
                    try {
                        accept.close();
                    }
                    catch (IOException ex2) {}
                    throw new IOException("Resource limited: too many open socket channels", cause);
                }
                n = 1L;
            }
        }
        finally {
            if (approver != null) {
                approver.request(-(request - n), of);
            }
        }
        return accept;
    }
    
    public final void close() throws IOException {
    }
    
    @InstrumentationMethod
    private int accept(FileDescriptor fileDescriptor, FileDescriptor fileDescriptor2, InetSocketAddress[] array) throws IOException {
        int accept = this.accept(fileDescriptor, fileDescriptor2, array);
        ResourceIdImpl of = ResourceIdImpl.of(fileDescriptor2);
        if (of != null) {
            ResourceRequest approver = ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(fileDescriptor2);
            long request = 0L;
            long n = 0L;
            try {
                try {
                    request = approver.request(1L, of);
                    if (request < 1L) {
                        throw new IOException("Resource limited: too many open file descriptors");
                    }
                }
                catch (ResourceRequestDeniedException cause) {
                    throw new IOException("Resource limited: too many open file descriptors", cause);
                }
                n = 1L;
            }
            finally {
                if (n == 0L) {
                    try {
                        ServerSocketChannelImplRMHooks.nd.close(fileDescriptor2);
                    }
                    catch (IOException ex) {}
                }
                else {
                    approver.request(-(request - 1L), of);
                }
            }
        }
        return accept;
    }
    
    @InstrumentationMethod
    public ServerSocketChannel bind(SocketAddress socketAddress, int n) throws IOException {
        ResourceIdImpl of = null;
        ResourceRequest approver = null;
        long request = 0L;
        if (this.getLocalAddress() == null) {
            of = ResourceIdImpl.of(socketAddress);
            approver = ApproverGroup.SOCKET_OPEN_GROUP.getApprover(this);
            request = approver.request(1L, of);
            if (request < 1L) {
                throw new ResourceRequestDeniedException("Resource limited: too many open socket channels");
            }
        }
        int n2 = 0;
        ServerSocketChannel bind = null;
        try {
            bind = this.bind(socketAddress, n);
            n2 = 1;
        }
        finally {
            if (approver != null) {
                approver.request(-(request - (long)n2), of);
            }
        }
        return bind;
    }
    
    abstract class NativeDispatcher
    {
        abstract void close(FileDescriptor p0) throws IOException;
    }
}
